using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Windows.Forms;

using GRV11; // Requires GRV11.cs or a "reference" to GRV11DLL.dll

using System.Drawing;
using System.Drawing.Imaging;

public class CCube
{
    protected float FScaleMultiplier = 50;

    protected float FXpos = 0f;
    protected float FYpos = 0f;
    protected float FZpos = 0f;

    protected float FXrot = 0f;
    protected float FYrot = 0f;
    protected float FZrot = 0f;

    protected float FXscale = 1f;
    protected float FYscale = 1f;
    protected float FZscale = 1f;

    public float ScaleMultiplier
    {
        get
        {
            return FScaleMultiplier;
        }

        set
        {
            FScaleMultiplier = value;
        }
    }

    public float Xpos
    {
        get
        {
            return FXpos;
        }

        set
        {
            FXpos = value;
        }
    }

    public float Ypos
    {
        get
        {
            return FYpos;
        }

        set
        {
            FYpos = value;
        }
    }

    public float Zpos
    {
        get
        {
            return FZpos;
        }

        set
        {
            FZpos = value;
        }
    }

    public float Xrot
    {
        get
        {
            return FXrot;
        }

        set
        {
            FXrot = value;
        }
    }

    public float Yrot
    {
        get
        {
            return FYrot;
        }

        set
        {
            FYrot = value;
        }
    }

    public float Zrot
    {
        get
        {
            return FZrot;
        }

        set
        {
            FZrot = value;
        }
    }

    public float Xscale
    {
        get
        {
            return FXscale;
        }

        set
        {
            FXscale = value;
        }
    }

    public float Yscale
    {
        get
        {
            return FYscale;
        }

        set
        {
            FYscale = value;
        }
    }

    public float Zscale
    {
        get
        {
            return FZscale;
        }

        set
        {
            FZscale = value;
        }
    }

    public float MaxXPos()
    {
        return FXpos * FXscale * FScaleMultiplier + 0.5f * FXscale * FScaleMultiplier;
    }

    public float MinXPos()
    {
        return FXpos * FXscale * FScaleMultiplier - 0.5f * FXscale * FScaleMultiplier;
    }

    public float MaxYPos()
    {
        return FYpos * FYscale * FScaleMultiplier + 0.5f * FYscale * FScaleMultiplier;
    }

    public float MinYPos()
    {
        return FYpos * FYscale * FScaleMultiplier - 0.5f * FYscale * FScaleMultiplier;
    }

    public float MaxZPos()
    {
        return FZpos * FZscale * FScaleMultiplier + 0.5f * FZscale * FScaleMultiplier;
    }

    public float MinZPos()
    {
        return FZpos * FZscale * FScaleMultiplier - 0.5f * FZscale * FScaleMultiplier;
    }

    public void DrawCube(GR PGR)
    {
        PGR.glMatrixMode(GR.GL_MODELVIEW);

        PGR.glPushMatrix();

        PGR.glTranslatef(FXpos * FXscale * FScaleMultiplier,
                         FYpos * FYscale * FScaleMultiplier,
                         FZpos * FZscale * FScaleMultiplier);

        PGR.glRotatef(FXrot, 1f, 0f, 0f);
        PGR.glRotatef(FYrot, 0f, 1f, 0f);
        PGR.glRotatef(FZrot, 0f, 0f, 1f);
        
        PGR.glBegin(GR.GL_QUADS);
        PGR.glColor3f(1f, 0f, 0f);
        PGR.glVertex3f(-0.5f * FXscale * FScaleMultiplier,
                       -0.5f * FYscale * FScaleMultiplier,
                       -0.5f * FZscale * FScaleMultiplier);
        PGR.glVertex3f(+0.5f * FXscale * FScaleMultiplier,
                       -0.5f * FYscale * FScaleMultiplier,
                       -0.5f * FZscale * FScaleMultiplier);
        PGR.glVertex3f(+0.5f * FXscale * FScaleMultiplier,
                       +0.5f * FYscale * FScaleMultiplier,
                       -0.5f * FZscale * FScaleMultiplier);
        PGR.glVertex3f(-0.5f * FXscale * FScaleMultiplier,
                       +0.5f * FYscale * FScaleMultiplier,
                       -0.5f * FZscale * FScaleMultiplier);

        PGR.glColor3f(0f, 1f, 0f);
        PGR.glVertex3f(-0.5f * FXscale * FScaleMultiplier,
                       -0.5f * FYscale * FScaleMultiplier,
                       +0.5f * FZscale * FScaleMultiplier);
        PGR.glVertex3f(+0.5f * FXscale * FScaleMultiplier,
                       -0.5f * FYscale * FScaleMultiplier,
                       +0.5f * FZscale * FScaleMultiplier);
        PGR.glVertex3f(+0.5f * FXscale * FScaleMultiplier,
                       +0.5f * FYscale * FScaleMultiplier,
                       +0.5f * FZscale * FScaleMultiplier);
        PGR.glVertex3f(-0.5f * FXscale * FScaleMultiplier,
                       +0.5f * FYscale * FScaleMultiplier,
                       +0.5f * FZscale * FScaleMultiplier);

        PGR.glColor3f(0f, 0f, 1f);
        PGR.glVertex3f(-0.5f * FXscale * FScaleMultiplier,
                       -0.5f * FYscale * FScaleMultiplier,
                       -0.5f * FZscale * FScaleMultiplier);
        PGR.glVertex3f(-0.5f * FXscale * FScaleMultiplier,
                       -0.5f * FYscale * FScaleMultiplier,
                       +0.5f * FZscale * FScaleMultiplier);
        PGR.glVertex3f(-0.5f * FXscale * FScaleMultiplier,
                       +0.5f * FYscale * FScaleMultiplier,
                       +0.5f * FZscale * FScaleMultiplier);
        PGR.glVertex3f(-0.5f * FXscale * FScaleMultiplier,
                       +0.5f * FYscale * FScaleMultiplier,
                       -0.5f * FZscale * FScaleMultiplier);

        PGR.glColor3f(1f, 1f, 0f);
        PGR.glVertex3f(+0.5f * FXscale * FScaleMultiplier,
                       -0.5f * FYscale * FScaleMultiplier,
                       -0.5f * FZscale * FScaleMultiplier);
        PGR.glVertex3f(+0.5f * FXscale * FScaleMultiplier,
                       -0.5f * FYscale * FScaleMultiplier,
                       +0.5f * FZscale * FScaleMultiplier);
        PGR.glVertex3f(+0.5f * FXscale * FScaleMultiplier,
                       +0.5f * FYscale * FScaleMultiplier,
                       +0.5f * FZscale * FScaleMultiplier);
        PGR.glVertex3f(+0.5f * FXscale * FScaleMultiplier,
                       +0.5f * FYscale * FScaleMultiplier,
                       -0.5f * FZscale * FScaleMultiplier);

        PGR.glColor3f(0f, 1f, 1f);
        PGR.glVertex3f(-0.5f * FXscale * FScaleMultiplier,
                       -0.5f * FYscale * FScaleMultiplier,
                       -0.5f * FZscale * FScaleMultiplier);
        PGR.glVertex3f(-0.5f * FXscale * FScaleMultiplier,
                       -0.5f * FYscale * FScaleMultiplier,
                       +0.5f * FZscale * FScaleMultiplier);
        PGR.glVertex3f(+0.5f * FXscale * FScaleMultiplier,
                       -0.5f * FYscale * FScaleMultiplier,
                       +0.5f * FZscale * FScaleMultiplier);
        PGR.glVertex3f(+0.5f * FXscale * FScaleMultiplier,
                       -0.5f * FYscale * FScaleMultiplier,
                       -0.5f * FZscale * FScaleMultiplier);

        PGR.glColor3f(1f, 1f, 1f);
        PGR.glVertex3f(-0.5f * FXscale * FScaleMultiplier,
                       +0.5f * FYscale * FScaleMultiplier,
                       -0.5f * FZscale * FScaleMultiplier);
        PGR.glVertex3f(-0.5f * FXscale * FScaleMultiplier,
                       +0.5f * FYscale * FScaleMultiplier,
                       +0.5f * FZscale * FScaleMultiplier);
        PGR.glVertex3f(+0.5f * FXscale * FScaleMultiplier,
                       +0.5f * FYscale * FScaleMultiplier,
                       +0.5f * FZscale * FScaleMultiplier);
        PGR.glVertex3f(+0.5f * FXscale * FScaleMultiplier,
                       +0.5f * FYscale * FScaleMultiplier,
                       -0.5f * FZscale * FScaleMultiplier);
        PGR.glEnd();

        PGR.glPopMatrix();
    }
}

public class Handler1
{
    private Form1 mForm = null;

    private GRVector3 mCameraPosition;
    private GRVector3 mCameraAngle;

    private bool   mCollide   = true;
    private bool[] mIsPressed = new bool[7];
    private bool   mJump      = false;
    private float  mJumpDir   = 1.0f;
    private float  mJumpY     = 0f;
    private float  mRot       = 0f;

    public void ClearIsPressed()
    {
        for (int index = 0; index < mIsPressed.Length; index++)
        {
            mIsPressed[index] = false;
        }
    }

    public int GetKeyIndex(Keys k)
    {
        switch (k)
        {
            case Keys.Up :
                return 1;
            case Keys.Down :
                return 2;
            case Keys.Left :
                return 3;
            case Keys.Right :
                return 4;
            case Keys.ControlKey :
                return 5;
            case Keys.Escape :
                return 6;
            default :
                return 0;
        }
    }

    public bool IsKeyPressed(Keys k)
    {
        return mIsPressed[GetKeyIndex(k)];
    }

    public Handler1(Form1 form, GRControl grControl)
    {
        /*
        this.mGRMesh = new GRMesh();
        this.mGRTexture = new GRTexture();
        */

        mForm = form;

        this.mCameraPosition.Set(0.0f, 25.0f, 50.0f);
        this.mCameraAngle.Set(0.0f, 0.0f, 0.0f);

        ClearIsPressed();

        Cursor.Hide();
        Cursor.Position = new System.Drawing.Point(mForm.Left + grControl.Left + (grControl.Width / 2),
                                                   mForm.Top + grControl.Top + (grControl.Height / 2));
    }

    public void OpenGLStarted(GRControl grControl)
    {
        /*
        GR gr = grControl.GetGR();
        if (null == gr) return;

        this.mGRMesh = new GRMesh();
        this.mGRMesh.BuildSimpleCube(150.0f);

        this.mGRTexture = new GRTexture();
        this.mGRTexture.CreateCheckerboardTexture(gr, 256, 256);
        */
    }

    public void Paint(object sender, PaintEventArgs e)
    {
        int FPS = System.Environment.TickCount;

        if (null == sender) return;
        if (false == (sender is GRControl)) return;

        GRControl grControl = (sender as GRControl);
        GR gr = grControl.GetGR();

        // create and set cubes
        CCube cubeA = new CCube();
        CCube cubeB = new CCube();

        cubeA.ScaleMultiplier = 50f;
        cubeB.ScaleMultiplier = 50f;

        cubeA.Xscale = 0.5f;
        cubeA.Yscale = 0.5f;
        cubeA.Zscale = 2.0f;

        cubeB.Xscale = 0.5f;
        cubeB.Yscale = 0.6f;
        cubeB.Zscale = 1.0f;

        cubeA.Zpos = -0.6f;
        cubeB.Zpos = +0.5f;

        mRot = mRot + 0.1f;
        cubeA.Zrot = mRot; // test Yrot
        cubeB.Zrot = mRot;

        // do a JUMP
        if (!(mJump))
        {
            mJump = IsKeyPressed(Keys.ControlKey);
            if (mJump)
            {
                mJumpDir = 1.0f;
                mJumpY   = 0f;
            }
        }
        if (mJump)
        {
            mCameraPosition.mY += mJumpDir;
            mJumpY += mJumpDir;

            mJumpDir -= 0.05f;
            if (mJumpY < 0)
            {
                mCameraPosition.mY -= mJumpY;
                mJump = false;
            }
        }

        // do a MOVEMENT
        double dX = mCameraPosition.mX;
        double dY = mCameraAngle.mY / 180 * Math.PI;
        double dZ = mCameraPosition.mZ;
        if (IsKeyPressed(Keys.Up))
        {
            dX += 0.5f * Math.Sin(dY);
            dZ -= 0.5f * Math.Cos(dY);
        }
        if (IsKeyPressed(Keys.Down))
        {
            dX -= 0.5f * Math.Sin(dY);
            dZ += 0.5f * Math.Cos(dY);
        }
        if (IsKeyPressed(Keys.Left))
        {
            dX -= 0.5f * Math.Cos(dY);
            dZ -= 0.5f * Math.Sin(dY);
        }
        if (IsKeyPressed(Keys.Right))
        {
            dX += 0.5f * Math.Cos(dY);
            dZ += 0.5f * Math.Sin(dY);
        }
        mCameraPosition.mX = (float)dX;
        mCameraPosition.mZ = (float)dZ;

        // most simple collision detection
        bool CollideA = false;
        bool CollideB = false;
        if (!(mJump))
        {
            // Console.WriteLine("AY: " + cubeA.MaxXPos().ToString() + " <= " + mCameraPosition.mY.ToString());
            if (cubeA.MaxYPos() <= mCameraPosition.mY)
            {
                // Console.WriteLine("AX: " + cubeA.MinXPos().ToString() + " <= " + mCameraPosition.mX + " <= " + cubeA.MaxXPos());
                // Console.WriteLine("AZ: " + cubeA.MinZPos().ToString() + " <= " + mCameraPosition.mZ + " <= " + cubeA.MaxZPos());
                // Console.WriteLine("BX: " + cubeB.MinXPos().ToString() + " < " + mCameraPosition.mX + " < " + cubeB.MaxXPos());
                // Console.WriteLine("BZ: " + cubeB.MinZPos().ToString() + " < " + mCameraPosition.mZ + " < " + cubeB.MaxZPos());

                CollideA = (cubeA.MinXPos() <= mCameraPosition.mX) && (cubeA.MaxXPos() >= mCameraPosition.mX);
                if (CollideA)
                {
                    CollideA = (cubeA.MinZPos() <= mCameraPosition.mZ) && (cubeA.MaxZPos() >= mCameraPosition.mZ);
                }
                CollideB = (cubeB.MinXPos() <= mCameraPosition.mX) && (cubeB.MaxXPos() >= mCameraPosition.mX);
                if (CollideB)
                {
                    CollideB = (cubeB.MinZPos() <= mCameraPosition.mZ) && (cubeB.MaxZPos() >= mCameraPosition.mZ);
                }
                mCollide = (CollideA || CollideB);
            }
            if (!(mCollide))
            {
                mCameraPosition.mY += mJumpDir;
                mJumpY += mJumpDir;

                mJumpDir -= 0.05f;
            }
        }

        // do a RESET
        if (IsKeyPressed(Keys.Escape))
        {
            mCollide = true;
            mJump    = false;

            this.mCameraPosition.Set(0.0f, 25.0f, 50.0f);
            this.mCameraAngle.Set(0.0f, 0.0f, 0.0f);
        }

        int clientWidth = grControl.ClientRectangle.Width;
        int clientHeight = grControl.ClientRectangle.Height;
        if (clientWidth <= 0)
        {
            clientWidth = 1;
        }
        if (clientHeight <= 0)
        {
            clientHeight = 1;
        }

        // Viewport - defines drawing range
        gr.glViewport(0, 0, clientWidth, clientHeight);

        // Basic rendering conditions
        gr.glEnable(GR.GL_DEPTH_TEST);
        //gr.glEnable(GR.GL_CULL_FACE);
        //gr.glCullFace(GR.GL_BACK);
        /*
        gr.glDepthFunc(GR.GL_LEQUAL);
        gr.glFrontFace(GR.GL_CCW);
         */

        gr.glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
        gr.glClear(GR.GL_COLOR_BUFFER_BIT | GR.GL_DEPTH_BUFFER_BIT);
 
        gr.glMatrixMode(GR.GL_PROJECTION);
        gr.glLoadIdentity();

        gr.gluPerspective(60.0f, ((double)(clientWidth) / (double)(clientHeight)), 0.1f, 64000f);
 
        gr.glMatrixMode(GR.GL_MODELVIEW);
        gr.glLoadIdentity();
        gr.glScalef(1f, 1f, 1f);

        //!!! user movement
        gr.glRotatef(mCameraAngle.mX, 1, 0, 0);
        gr.glRotatef(mCameraAngle.mY, 0, 1, 0);
        gr.glRotatef(mCameraAngle.mZ, 0, 0, 1);

//        gr.glRotatef(0, 0f, 1f, 0f);
        gr.glTranslatef((-(this.mCameraPosition.mX)), (-(this.mCameraPosition.mY)), (-(this.mCameraPosition.mZ)));
        //!!! user movement

        cubeA.DrawCube(gr);
        cubeB.DrawCube(gr);

        gr.wglSwapBuffers(grControl.GetHDC());

        FPS = 1000 / (System.Environment.TickCount - FPS + 1);
        mForm.Text = "FPS=" + FPS.ToString() +
                     " aX=" + mCameraAngle.mX.ToString() +
                     " aY=" + mCameraAngle.mY.ToString() +
                     " pX=" + mCameraPosition.mX.ToString() +
                     " pY=" + mCameraPosition.mY.ToString() +
                     " pZ=" + mCameraPosition.mZ.ToString() +
                     " Ju=" + mJump.ToString() + 
                     " Co=" + mCollide.ToString() +
                     " aC=" + CollideA.ToString() +
                     " bC=" + CollideB.ToString();
    }

    public void KeyPress(object sender, KeyPressEventArgs e)
    {
        // empty
    }

    public void KeyDown(object sender, KeyEventArgs e)
    {
        mIsPressed[GetKeyIndex(e.KeyCode)] = true;
    }

    public void KeyUp(object sender, KeyEventArgs e)
    {
        mIsPressed[GetKeyIndex(e.KeyCode)] = false;
    }

    public void MouseDown(object sender, MouseEventArgs e)
    {
        // empty
    }

    public void MouseUp(object sender, MouseEventArgs e)
    {
        // empty
    }

    public void MouseMove(object sender, MouseEventArgs e)
    {
        if (null == sender) return;
        if (false == (sender is GRControl)) return;
        GRControl grControl = (sender as GRControl);

        Point lCursorPos = Cursor.Position;
        lCursorPos.X -= mForm.Left + grControl.Left + (grControl.Width / 2);
        lCursorPos.Y -= mForm.Top + grControl.Top + (grControl.Height / 2);

        if ((Math.Abs(lCursorPos.X) > 10) || (Math.Abs(lCursorPos.Y) > 10))
        {
            this.mCameraAngle.mX += 0.05f * (float)(lCursorPos.Y);
            this.mCameraAngle.mY += 0.05f * (float)(lCursorPos.X);

            this.mCameraAngle.mY %= 360;
            if (this.mCameraAngle.mY < 0)
            {
                this.mCameraAngle.mY += 360;
            }

            if (mCameraAngle.mX < -45)
            {
                mCameraAngle.mX = -45;
            }
            else
            {
                if (mCameraAngle.mX > 45)
                {
                    mCameraAngle.mX = 45;
                }
            }

            Cursor.Position = new System.Drawing.Point(mForm.Left + grControl.Left + (grControl.Width / 2),
                                                       mForm.Top + grControl.Top + (grControl.Height / 2));
        }
    }

    public void MouseWheel(object sender, MouseEventArgs e)
    {
        // empty
    }
}