using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using GRV11; // Requires GRV11.cs source code or a "reference" to GRV11DLL.dll

public partial class Form1 : Form
{
    public GRControl mGRControl;
    public Handler1 mGRExampleHandler1;
    private System.Windows.Forms.Timer mTimer;

    public Form1()
    {
        InitializeComponent();

        this.mGRExampleHandler1 = new Handler1(this, mGRControl);
        this.mGRControl.OpenGLStarted += new GRControl.DelegateOpenGLStarted(this.mGRExampleHandler1.OpenGLStarted);
        this.mGRControl.KeyDown += new KeyEventHandler(this.mGRExampleHandler1.KeyDown);
        this.mGRControl.KeyUp += new KeyEventHandler(this.mGRExampleHandler1.KeyUp);
        this.mGRControl.MouseDown += new MouseEventHandler(this.mGRExampleHandler1.MouseDown);
        this.mGRControl.MouseUp += new MouseEventHandler(this.mGRExampleHandler1.MouseUp);
        this.mGRControl.MouseMove += new MouseEventHandler(this.mGRExampleHandler1.MouseMove);
        this.mGRControl.MouseWheel += new MouseEventHandler(this.mGRExampleHandler1.MouseWheel);
        this.mGRControl.Paint += new PaintEventHandler(this.mGRExampleHandler1.Paint);
        this.mGRControl.KeyPress += new KeyPressEventHandler(this.mGRExampleHandler1.KeyPress);

        this.mTimer = new System.Windows.Forms.Timer();
        this.mTimer.Interval = 10; // 10-millisecond timer
        this.mTimer.Tick += new EventHandler(PrivateTimerTickEventHandler);
        this.mTimer.Start();

        // Set focus to the control so that it can immediately accept input
        this.mGRControl.Focus();

        // Also, whenever the form becomes activated, set focus to the main
        // control on the form.  The following sets up an event handler for
        // that purpose.
        this.Activated += new EventHandler(PrivateActivatedEventHandler);

        // We want to preview dialog keys (most importantly, the cursor
        // keys: up, down, right, left) so we can forward such events to
        // the appropriate child control.
        this.KeyPreview = true;
    }

    void PrivateTimerTickEventHandler(object sender, EventArgs e)
    {
        if (false == DesignMode)
        {
            this.mGRControl.Invalidate();
        }
    }

    private void PrivateActivatedEventHandler(object sender, EventArgs e)
    {
        // When this form becomes activated, after some time of not
        // being active, set input focus to the main control on the form.
        this.mGRControl.Focus();
    }

    // Cursor keys (up,down,left,right) need to be specially captured
    // and forwarded to the control.
    // CAUTION: The KeyPreview property of this Form must be set to 'true' 
    // for the following method to be called.
    protected override bool ProcessDialogKey(Keys keyData)
    {
        if
            (
               (keyData == Keys.Up)
            || (keyData == Keys.Down)
            || (keyData == Keys.Left)
            || (keyData == Keys.Right)
            )
        {
            KeyEventArgs e = new KeyEventArgs(keyData);
            this.mGRExampleHandler1.KeyDown(this.mGRControl, e);
            return (true);
        }

        return base.ProcessDialogKey(keyData);
    }
}